@@ -26,13 +26,9 @@
 #include <X11/Xproto.h>
 #include <gio/gio.h>
 
-#include "image.h"
-
 #include "wallpaper_manager.h"
 #include "gnome_wallpaper_manager.h"
 
-#define XFDESKTOP_SELECTION_FMT "XFDESKTOP_SELECTION_%d"
-
 typedef struct {
     gint16 r;
     gint16 g;
@@ -53,7 +49,7 @@ rstto_gnome_wallpaper_manager_finalize (GObject *object);
 
 static GObjectClass *parent_class = NULL;
 
-static RsttoGnomeWallpaperManager *gnome_wallpaper_manager_object;
+static RsttoWallpaperManager *gnome_wallpaper_manager_object;
 
 struct _RsttoGnomeWallpaperManagerPriv
 {
@@ -66,7 +62,7 @@ enum
 };
 
 static gint 
-rstto_gnome_wallpaper_manager_configure_dialog_run (RsttoWallpaperManager *self, RsttoImage *image)
+rstto_gnome_wallpaper_manager_configure_dialog_run (RsttoWallpaperManager *self, GFile *file)
 {
     RsttoGnomeWallpaperManager *manager = RSTTO_GNOME_WALLPAPER_MANAGER (self);
     gint response = GTK_RESPONSE_OK;
@@ -103,7 +99,7 @@ rstto_gnome_wallpaper_manager_check_running (RsttoWallpaperManager *self)
 }
 
 static gboolean
-rstto_gnome_wallpaper_manager_set (RsttoWallpaperManager *self, RsttoImage *image)
+rstto_gnome_wallpaper_manager_set (RsttoWallpaperManager *self, GFile *file)
 {
     RsttoGnomeWallpaperManager *manager = RSTTO_GNOME_WALLPAPER_MANAGER (self);
 
@@ -209,7 +205,7 @@ rstto_gnome_wallpaper_manager_finalize (GObject *object)
  *
  * Singleton
  */
-RsttoGnomeWallpaperManager *
+RsttoWallpaperManager *
 rstto_gnome_wallpaper_manager_new (void)
 {
     if (gnome_wallpaper_manager_object == NULL)
