@@ -30,10 +30,8 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libexif/exif-data.h>
 
-#include "image.h"
 #include "image_list.h"
 #include "settings.h"
-#include "picture_viewer.h"
 #include "main_window.h"
 
 
@@ -46,7 +44,7 @@ typedef struct {
     gint argc;
     gchar **argv;
     gint iter;
-    RsttoMainWindow *window;
+    GtkWidget *window;
     gboolean open_entire_folder;
 } RsttoOpenFiles;
 
@@ -119,11 +117,17 @@ main(int argc, char **argv)
         rof.argc = argc;
         rof.argv = argv;
     	rof.iter = 1;
-        rof.window = RSTTO_MAIN_WINDOW (window);
         rof.open_entire_folder = rstto_settings_get_boolean_property (settings, "open-entire-folder");
+        rof.window = window;
 
         g_idle_add ((GSourceFunc )cb_rstto_open_files, &rof);
 
+        if (TRUE == rstto_settings_get_boolean_property (
+                    settings,
+                    "maximize-on-startup"))
+        {
+            gtk_window_maximize (GTK_WINDOW(window));
+        }
     }
 
     g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
@@ -143,6 +147,7 @@ main(int argc, char **argv)
 static gboolean
 cb_rstto_open_files (RsttoOpenFiles *rof)
 {
+    GFileType file_type;
     GFile *file, *p_file, *child_file;
     GFileInfo *file_info;
     const gchar *content_type, *filename;
@@ -156,14 +161,50 @@ cb_rstto_open_files (RsttoOpenFiles *rof)
             file = g_file_new_for_commandline_arg (rof->argv[rof->iter]);
             if (file)
             {
-                file_info = g_file_query_info (file, "standard::content-type", 0, NULL, NULL);
+                if (rof->open_entire_folder) 
+                {
+                    file_info = g_file_query_info (file, "standard::content-type,standard::type", 0, NULL, NULL);
+                }
+                else
+                {
+                    file_info = g_file_query_info (file, "standard::content-type", 0, NULL, NULL);
+                }
+
                 if (file_info)
                 {
                     content_type = g_file_info_get_attribute_string (file_info, "standard::content-type");
 
                     if (strncmp (content_type, "image/", 6) == 0)
                     {
-                        rstto_image_list_add_file (rof->image_list, file, NULL);
+                        if (rstto_image_list_add_file (rof->image_list, file, NULL) == TRUE)
+                        {
+                            rstto_main_window_add_file_to_recent_files (file);
+                        }
+                    }
+
+                    if (rof->open_entire_folder) 
+                    {
+                        file_type = g_file_info_get_file_type(file_info);
+                        if (file_type == G_FILE_TYPE_DIRECTORY)
+                        {
+                            rstto_main_window_add_file_to_recent_files (file);
+
+                            file_enumarator = g_file_enumerate_children (file, "standard::*", 0, NULL, NULL);
+                            for(file_info = g_file_enumerator_next_file (file_enumarator, NULL, NULL); file_info != NULL; file_info = g_file_enumerator_next_file (file_enumarator, NULL, NULL))
+                            {
+                                filename = g_file_info_get_name (file_info);
+                                content_type  = g_file_info_get_content_type (file_info);
+                                child_file = g_file_get_child (file, filename);
+
+                                if (strncmp (content_type, "image/", 6) == 0)
+                                {
+                                    rstto_image_list_add_file (rof->image_list, child_file, NULL);
+                                }
+
+                                g_object_unref (child_file);
+                                g_object_unref (file_info);
+                            }
+                        }
                     }
                 }
             }
@@ -176,18 +217,29 @@ cb_rstto_open_files (RsttoOpenFiles *rof)
         file = g_file_new_for_commandline_arg (rof->argv[rof->iter]);
         if (file)
         {
-            file_info = g_file_query_info (file, "standard::content-type", 0, NULL, NULL);
+            file_info = g_file_query_info (file, "standard::content-type,standard::type", 0, NULL, NULL);
             if (file_info)
             {
                 content_type = g_file_info_get_attribute_string (file_info, "standard::content-type");
+                file_type = g_file_info_get_file_type (file_info);
 
                 if (strncmp (content_type, "image/", 6) == 0)
                 {
-                    rstto_image_list_add_file (rof->image_list, file, NULL);
+                    if (rstto_image_list_add_file (rof->image_list, file, NULL) == TRUE)
+                    {
+                        rstto_main_window_add_file_to_recent_files (file);
+                    }
                 }
             }
         }
-        p_file = g_file_get_parent (file);
+        if (file_type != G_FILE_TYPE_DIRECTORY) {
+            p_file = g_file_get_parent (file);
+        }
+        else
+        {
+            rstto_main_window_add_file_to_recent_files (file);
+            p_file = file;
+        }
         file_enumarator = g_file_enumerate_children (p_file, "standard::*", 0, NULL, NULL);
         for(file_info = g_file_enumerator_next_file (file_enumarator, NULL, NULL); file_info != NULL; file_info = g_file_enumerator_next_file (file_enumarator, NULL, NULL))
         {
